/*
 * main.c
 *
 * Created: 05.01.2015 09:00:00
 *  Author: FKleineKappenberg
 *  
 *  Im Rahmen dieser Software wird der Messchip zunchst Pro-
 *  grammiert und anschlieend mithilfe des durch den
 *  externen Interrupt der INTN-Leitung aufgerufenen
 *  Routine jeweils Messwerte fr Strom und Spannung aus dem
 *  Messchip ausgelesen.
 *  Bei gestecktem Jumper JP_1 wird Mithilfe einer durch einen 
 *  Timerinterrupt ausgelsten Routine in regelmigen Abstnden
 *  der Messchip fr eine kurze Zeit umkonfiguriert und die 
 *  Temperaturmessung durchgefhrt.
 *  Ist der Jumper an JP_2 gesteckt, erfolgt eine Mittelwert-
 *  Bildung der Strom- und Spannungsmesswerte.
 *  Alle 1000 Messungen von Strom und Spannung wird im Rahmen
 *  des zuerst genannten Interrupthandlers eine Ausgabe der 
 *  Messwerte ber RS-232 vorgenommen, um diese mithilfe eines
 *  Terminalprogramms am Computer prfen zu knnen.
 *  Diese Ausgabe ist nur fr Testzwecke gedacht, da durch
 *  ihre Ausfhrung im Interrupthandler wrendessen
 *  keine Messwerte aus dem Messchip ausgelesen werden knnen.
 *   
 */ 

/********************Includes********************/

#include <avr32/io.h>
//bindet microcontrollerspezifische I/O Definitionen ein

#include <stdint.h>
//bindet Integertypen ein
	
#include <asf.h>
//bindet das ASF ein

#include "intc.h"
//bindet den Treiber des Interruptcontrollers aus
//dem ASF ein zugehrige verwendete Routinen:
	//void INTC_init_interrupts(void);
		//initialisiert den Interruptcontroller
	//void INTC_register_interrupt(__int_handler handler, 
	//uint32_t irq, uint32_t int_level);
		//registriert Interrupt im Interruptcontroller
		//unter Angabe der Adresse des Interrupthandlers,
		//der Nummer der Interruptleitung
		//und der Prioritt [15, S. 183ff.]
		
#include "usart.h"  
//bindet den USART-Treiber aus dem ASF ein
//zugehrige verwendete Routinen:
	//extern int usart_init_rs232_tx_only(
	//volatile avr32_usart_t *usart, 
	//const usart_options_t *opt, long pba_hz);
		//initialisiert eine USART fr
		//RS-232 (nur senden)
	//extern int usart_putchar(
	//volatile avr32_usart_t *usart, int c);
		//sendet ein Zeichen ber eine USART
	//extern void usart_write_line(
	//volatile avr32_usart_t *usart, const char *string);
		//sendet eine Zeichenkette ber eine USART
	
#include "canif.h" 
//bindet den CAN-Interface-Treiber aus dem ASF ein

#include "can.h"
//bindet den CAN-Service aus dem ASF ein
	
#include "math.h"
//Mathematische Rechenoperationen einbinden
	
/********************Funktionsdeklarationen********************/

/*+++++Microcontrollerinitialisierung+++++*/
void mcu_init(void);	//Initialisierung des Microcontrollers
void clk_init(void);	//Initialisierung des 16 MHz CPU-Takts
void gpio_init(void);	//Initialisierung der GPIO-Pins
void ast_init(void);	//Konfiguration und Start des AST
void gclk_init(void);	//Initialisierung des 8 MHz Messchiptakts
void usart_init(void);	//initialisiere USART2 fr RS-232
void CANBUS_init(void); //initialisiere fr CAN-BUS

/*+++++allgemeine Hilfsroutinen+++++*/
void wait_cycles(uint32_t c);		 
	//Warte c Taktzyklen
void global_interrupt_disable(void);
	//aktiviere Interrupts global 
void global_interrupt_enable(void);
	 //deaktiviere Interrupts global 

/*+++++Hilfsfunktionen fr den GPIO-Controller+++++*/ 
void gpio_drive(uint8_t port, uint8_t portpin); 
	//treibe Pin eines Ports mit dem GPIO-Controller
void gpio_drive_not(uint8_t port, uint8_t portpin); 
	//treibe Pin nicht mit dem GPIO-Controller
void gpio_set(uint8_t port, uint8_t portpin); 
	//setze Pin auf High-Pegel (5V)
void gpio_toggle(uint8_t port, uint8_t portpin);
	//toggle Pin
void gpio_clear(uint8_t port, uint8_t portpin); 
	//setze Pin auf Low-Pegel (0V)
uint8_t gpio_read(uint8_t port, uint8_t portpin); 
	//lese Pegel an Pin aus und gebe fr High-Pegel eine
	//1 zurck, sonst eine 0
void gpio_enable_pullup(uint8_t port, uint8_t portpin); 
	//aktiviere Pullup-Widerstand fr Pin
void gpio_assign_function(uint8_t port, uint8_t portpin,
char func);
	//weise Pin die Peripheriefunktion 'a', 'b', 'c', 'd', 'e'
	//oder 'f' zu (vgl. Datenblatt)
void gpio_enable_interrupt(uint8_t port, uint8_t portpin, 
char mode);
	//aktiviere Interrupt im Modus 'c'=pin change,
	//'r'=rising edge,'f'=falling edge fr einen Pin 
void gpio_disable_interrupt(uint8_t port, uint8_t portpin); 
	//deaktiviere Interrupt fr einen Pin
void gpio_clear_interrupt_flag(uint8_t port, uint8_t portpin); 
	//lschen der Interruptflag fr einen Pin

/*+++++Hilfsroutinen des AST+++++*/
void ast_interrupt_enable(void); 
	//aktiviere den periodischen Interrupt 0 des AST
void ast_interrupt_disable(void); 
	//deaktiviere den periodischen Interrupt 0 des AST
void ast_clear_interrupt_flag(void); 
	//lschen der Interruptflag des periodischen Interrupts 0


void print_int32(int32_t i); 
	//gebe int32_t-Wert ber die RS-232-Schnittstelle aus


/*+++++CAN-BUS-Kommunikation+++++*/
// Local allocation for MOB buffer in HSB_RAM
volatile can_msg_t mob_ram_ch0[NB_MOB_CHANNEL] __attribute__((section (".hsb_ram_loc")));
//Call Back prototype TX
void can_out_callback_channel0_tx(uint8_t handle, uint8_t event);
//Call Back prototype RX
void can_out_callback_channel0_rx(uint8_t handle, uint8_t event);
//Local function to prepare TX buffers
void can_example_prepare_data(uint8_t mode);
//Messageblock function
void messageblock (uint32_t ident, uint32_t mask, U64 can_data);

/*+++++Interrupthandler+++++*/
void intn_int_handler(void); 
	//Interrupthandler fr externen Interrupt an INTN-Pin 
	//(Auslesen von Strom-/Spannungsmesswert)
void ast_int_handler(void); 
	//Interrupthandler fr Timerinterrupt (Temperaturmessung)
void calk_int_handler(void); 


/*************Pin- und Portbezeichnungen***********************/

//Die Pinnummer am jeweiligen Port kann durch Modulo 32 
//berechnet werden (32 Pins pro Port)
//Die Definitionen AVR_32_PIN_PC00 usw. sind durch Atmelstudio 
//vorgegeben

/*+++++CAN TXD und RXD (transmit, receive)+++++*/
#define CANIF_TXD AVR32_PIN_PC00%32 
#define CANIF_RXD AVR32_PIN_PC01%32 

/*+++++USART2, RS-232 TXD und RXD (transmit, receive)+++++*/
#define USART2_RXD AVR32_PIN_PC02%32 
#define USART2_TXD AVR32_PIN_PC03%32 

/*+++++LEDs und Jumper+++++*/
#define LED_G1 AVR32_PIN_PC04%32 //LED green 1
#define LED_G2 AVR32_PIN_PC05%32 //LED green 2
#define LED_G3 AVR32_PIN_PC06%32 //LED green 3
#define LED_Y1 AVR32_PIN_PC07%32 //LED yellow 1
#define LED_Y2 AVR32_PIN_PC11%32 //LED yellow 2
#define LED_Y3 AVR32_PIN_PC12%32 //LED yellow 3
#define LED_R1 AVR32_PIN_PC13%32 //LED red 1
#define LED_R2 AVR32_PIN_PC14%32 //LED red 2
#define JP_1 AVR32_PIN_PC15%32 //Jumper 1
#define JP_2 AVR32_PIN_PC16%32 //Jumper 2
#define JP_3 AVR32_PIN_PC17%32 //Jumper 3
#define JP_4 AVR32_PIN_PC18%32 //Jumper 4
#define JP_5 AVR32_PIN_PD07%32 //Jumper 5

/*+++++Messchip+++++*/
#define GCLK0_OUT AVR32_PIN_PD22%32 
//Clock-Leitung
#define ihm_intn_pin AVR32_PIN_PD14 
//Pin der INTN-Leitung
#define ihm_sdat_portpin AVR32_PIN_PD12%32 
//Portpin der SDAT-Leitung
#define ihm_sclk_portpin AVR32_PIN_PD13%32 
//Portpin der SCLK-Leitung
#define ihm_intn_portpin ihm_intn_pin%32 
//Portpin der INTN-Leitung

/*+++++Ports+++++*/
#define ihm_port 3 
//Kommunikation und Takt des Messchips an PORTD
#define portc 2 
//smtliche Peripheriefunktionen an PORTC (ausnahme JP_5)

//Die in obigen Definitionen gettigte Berechnung von Ports 
//und Pins knnte hierbei auch in die entsprechenden GPIO-
//Hilfsroutinen ausgelagert werden, jedoch msste diese 
//zustzliche Berechnung im laufenden Betrieb stndig neu 
//durchgefhrt werden, was Zeit kostet.


/****Definitionen und Makros fr die MesschipKommunikation****/

//CAN TX und RX Strukturen
can_mob_t rx_mob_msg;
can_mob_t tx_mob_msg;



//Makro fr das Warten zweier Taktzyklen (125 ns) durch 
//Auslesen eines ungenutzten Systemregisters 
int delay;
#define wait_2_cycles {delay=__builtin_mfsr(0x0000012C);delay=__builtin_mfsr(0x0000012C);}




/******************Implementierung der Routinen******************/

/*+++++Microcontrollerinitialisierung+++++*/
void mcu_init(void)
{
	clk_init();
	gpio_init();
	gclk_init();
	ast_init();
	usart_init();
	CANBUS_init();
}

void clk_init(void)
{
	//Initialisierung des Oszillators 0 mit dem 16 MHz Quarz 
	//als CPU-Taktquelle
	AVR32_SCIF.unlock = 0xAA000024; 
		//unlock OSCCTRL[0]-Register (Offset 0x0024)
	
	//Starte den Oscillator 0 mit folgenden Einstellungen:
	//Automatic Gain Control aktiv, Gain 3, 
	//Einschwingzeit 2^15 Zyklen [15, S. 103]
	AVR32_SCIF.oscctrl[0] = 0x0001040D; 
	while(
	(AVR32_SCIF.pclksr & AVR32_SCIF_PCLKSR_OSC0RDY_MASK)==0); 
		//warte bis Oszillator bereit und stabil
	AVR32_PM.unlock = 0xAA000000; 
		//unlock MCCTRL-Register (Offset 0x0000) 
		//im Power Manager 
	AVR32_PM.mcctrl = 0x00000001; 
		//nutze Osc0 als CPU-Taktquelle
}

void gpio_init(void)
{
	//Aktivierung der Treiber fr die LEDs
	gpio_drive(portc, LED_G1);
	gpio_drive(portc, LED_G2);
	gpio_drive(portc, LED_G3);
	gpio_drive(portc, LED_Y1);
	gpio_drive(portc, LED_Y2);
	gpio_drive(portc, LED_Y3);
	gpio_drive(portc, LED_R1);
	gpio_drive(portc, LED_R2);
	
	//Setzen von Pull-Up-Widerstnden
	gpio_enable_pullup(portc, JP_1);
	gpio_enable_pullup(portc, JP_2);
	gpio_enable_pullup(portc, JP_3);
	gpio_enable_pullup(portc, JP_4);
	gpio_enable_pullup(ihm_port, JP_5);
	
	//Konfiguration der seriellen Kommunikation mit dem 
	//Messchip
	gpio_drive(ihm_port, ihm_sclk_portpin);
	gpio_drive(ihm_port, ihm_sdat_portpin);
	//Aktivierung eines Interrupts am Pin der INTN-Leitung,
	//welcher auf eine fallende Taktflanke auslst
	gpio_enable_interrupt(ihm_port, ihm_intn_portpin,'f');
}

void ast_init(void)
{
	//Konfiguration der Generic Clock als Taktquelle des AST
	AVR32_SCIF.GCCTRL[2].oscsel = 7; 
		//whle den Prozessortakt als Taktquelle 
	AVR32_SCIF.GCCTRL[2].diven = 1; 
		//aktiviere Vorteiler
	AVR32_SCIF.GCCTRL[2].div = 0x00; 
		//whle das Teilerverhltnis 2 (2^(n+1) mit n=0)
	AVR32_SCIF.GCCTRL[2].cen = 1; 
		//Aktivierung der Generic Clock
	
	//Konfiguration des AST zur Auslsung eines Interrupts 
	//mit entsprechender Periodizitt
	//Die Periodendauer betrgt hierbei 1,053 Sekunden, 
	//da dies zunchst durch eine Division der 
	//Prozessortaktfrequenz um Zweierpotenzen ohne 
	//greren Aufwand zu realisieren war 
	AVR32_AST.CLOCK.cssel = 3; 
		//Auswahl der GCLK[2] 
		//als Taktquelle des Vorteilers
	while(AVR32_AST.SR.clkbusy); 
		//warte ab, bis Clock nicht mehr beschftigt
	AVR32_AST.CLOCK.cen = 1; 
		//aktiviere den Takt des Vorteilers
	while(AVR32_AST.SR.clkbusy); 
		//warte ab, bis Clock nicht mehr beschftigt
	while(AVR32_AST.SR.busy); 
		//warte ab, bis AST nicht mehr beschftigt
	AVR32_AST.PIR0.insel= 22; 
		//whle eine nderung des 22. Bit des Vorteilers 
		//als Auslser des periodischen Interrupts 0
	while(AVR32_AST.SR.busy); 
		//warte ab, bis AST nicht mehr beschftigt
	AVR32_AST.cr = 0x00000001; 
		//Starte Timer
	ast_interrupt_enable(); 
		//aktiviere den periodischen Interrupt 0
}

void gclk_init(void)
{
	//Konfiguration der Generic Clock 
	//als Taktquelle des Messchips
	gpio_assign_function(ihm_port, GCLK0_OUT,'d');	
		//Zuweisung des entsprechenden I/O-Pins
	AVR32_SCIF.GCCTRL[9].oscsel = 7;
		//whle den Prozessortakt als Taktquelle 
	AVR32_SCIF.GCCTRL[9].diven = 1; 
		//aktiviere Vorteiler
	AVR32_SCIF.GCCTRL[9].div = 0x00; 
		//whle das Teilerverhltnis 2 (2^(n+1) mit n=0)
	AVR32_SCIF.GCCTRL[9].cen = 1; 
		//Aktivierung der Generic Clock
}

void usart_init(void)
{
	//Initialisierung der USART fr die RS-232-Schnittstelle
	static const usart_options_t USART_OPTIONS =
	{
		.baudrate     = 56000, //Baudrate: 256 kBd
		.charlength   = 8, //Symbollnge: 8 Bit
		.paritytype   = USART_NO_PARITY, //keine Paritt
		.stopbits     = USART_1_STOPBIT, //1 Stoppbit
		.channelmode  = USART_NORMAL_CHMODE 
	};
	uint32_t MCU_CLK = 16000000; //Prozessortaktfrequenz
	gpio_assign_function(portc, USART2_TXD,'c'); 
	//Zuweisung der I/O-Pins
	gpio_assign_function(portc, USART2_RXD,'c');
	//ASF-Routine zur Initialisierung der USART2 
	//mit der o.g. Konfiguration fr die 
	//RS-232-Schnittstelle (nur senden)
	usart_init_rs232_tx_only(&AVR32_USART2,&USART_OPTIONS,
	MCU_CLK);
}

void CANBUS_init(void)
{
	uint32_t MCU_CLK = 16000000; //Prozessortaktfrequenz
	// Initialize the system clocks
	sysclk_init();

	// Switch the main clock to OSC0
	scif_configure_osc_crystalmode(SCIF_OSC0, MCU_CLK);
	scif_enable_osc(SCIF_OSC0, AVR32_SCIF_OSCCTRL0_STARTUP_2048_RCOSC, true);

	// Setup the generic clock for CAN
	scif_gc_setup(AVR32_SCIF_GCLK_CANIF,
	SCIF_GCCTRL_OSC0,
	AVR32_SCIF_GC_NO_DIV_CLOCK,
	0);
	
	scif_gc_setup(AVR32_SCIF_GCLK_GCLK0,
	SCIF_GCCTRL_OSC0,
	AVR32_SCIF_GC_NO_DIV_CLOCK,
	0);
	
	// Now enable the generic clock
	scif_gc_enable(AVR32_SCIF_GCLK_CANIF);
	scif_gc_enable(AVR32_SCIF_GCLK_GCLK0);
	//Enable USART Rx interrupt
	UART_USART->ier = AVR32_USART_IER_RXRDY_MASK;

	static const gpio_map_t CAN_GPIO_MAP = {
		{AVR32_CANIF_RXLINE_0_0_PIN, AVR32_CANIF_RXLINE_0_0_FUNCTION},
		{AVR32_CANIF_TXLINE_0_0_PIN, AVR32_CANIF_TXLINE_0_0_FUNCTION}
	};
	// Assign GPIO to CAN.
	gpio_enable_module(CAN_GPIO_MAP,
	sizeof(CAN_GPIO_MAP) / sizeof(CAN_GPIO_MAP[0]));

	//Pins enable
	gpio_assign_function(portc, CANIF_TXD,'e');
	gpio_assign_function(portc, CANIF_RXD,'e');
	
	//init_dbg_rs232(FPBA_HZ);
	
}

/*+++++Allgemeine Hilfsroutinen+++++*/
void wait_cycles(uint32_t c)
{
	//warten von c Taktzyklen mithilfe 
	//des Systemregisters AVR32_COUNT,
	//welches sich bei jedem Taktzyklus um 1 erhht
	__builtin_mtsr(AVR32_COUNT, 0);
		//setzen von AVR32_COUNT zu 0 
	int i = 0;
	while (i<c)
	{
		i=__builtin_mfsr(AVR32_COUNT);
	}
}

void global_interrupt_disable(void) 
{
	//deaktiviere interrupts global
	__builtin_ssrf(AVR32_SR_GM_OFFSET);
}

void global_interrupt_enable(void)
{
	usart_write_char(&AVR32_USART2, 4);
	//aktiviere interrupts global
	__builtin_csrf(AVR32_SR_GM_OFFSET);
}

/*+++++Hilfsroutinen fr den GPIO-Controller+++++*/
void gpio_drive(uint8_t port, uint8_t portpin)
{
	AVR32_GPIO.port[port].gpers = 1 << portpin; 
		//weise Pin dem GPIO-Controller zu
	AVR32_GPIO.port[port].oders = 1 << portpin; 
		//Aktiviere Treiber
	AVR32_GPIO.port[port].ovrc = 1 << portpin; 
		//setze Low-Pegel
}

void gpio_drive_not(uint8_t port, uint8_t portpin)
{
	AVR32_GPIO.port[port].gperc = 1 << portpin; 
		//Nehme Zuweisung zum GPIO-Controller zurck
	AVR32_GPIO.port[port].oderc = 1 << portpin; 
		//deaktiviere Treiber
}

void gpio_set(uint8_t port, uint8_t portpin)
{
	AVR32_GPIO.port[port].ovrs = 1 << portpin; 
		//setze Pin auf High-Pegel
}

void gpio_toggle(uint8_t port, uint8_t portpin)
{
	AVR32_GPIO.port[0].ovrt = 1 << portpin;
		//toggel Pin
}

void gpio_clear(uint8_t port, uint8_t portpin) 
{
	AVR32_GPIO.port[port].ovrc = 1 << portpin; 
		//setze Pin auf Low-Pegel
}

uint8_t gpio_read(uint8_t port, uint8_t portpin)
{
	//jederzeit mglich, auch wenn Pin durch
	//Peripheriemodul genutzt wird
	uint32_t mask = 0x00000000;
	mask = 1 << portpin;
	if ((AVR32_GPIO.port[port].pvr & mask)== 0)
	return 0;
	else
	return 1;
	//gebe
}

void gpio_assign_function(uint8_t port, uint8_t portpin, 
char func)
{

	AVR32_GPIO.port[port].gperc = 1 << portpin;
		//Nehme eventuelle Zuweisung des Pins an
		//den GPIO-Controller zurck
		
	switch(func)
	{
		case 'a':	
		    AVR32_GPIO.port[port].pmr0c = 1 << portpin;
		    AVR32_GPIO.port[port].pmr1c = 1 << portpin;
		    AVR32_GPIO.port[port].pmr2c = 1 << portpin;
		    break;
		case 'b':	
		    AVR32_GPIO.port[port].pmr0s = 1 << portpin;
		    AVR32_GPIO.port[port].pmr1c = 1 << portpin;
		    AVR32_GPIO.port[port].pmr2c = 1 << portpin;
		    break;
		case 'c':
		    AVR32_GPIO.port[port].pmr0c = 1 << portpin;
		    AVR32_GPIO.port[port].pmr1s = 1 << portpin;
		    AVR32_GPIO.port[port].pmr2c = 1 << portpin;
		    break;
		case 'd':	
		    AVR32_GPIO.port[port].pmr0s = 1 << portpin;
		    AVR32_GPIO.port[port].pmr1s = 1 << portpin;
		    AVR32_GPIO.port[port].pmr2c = 1 << portpin;
		    break;
		case 'e':	
		    AVR32_GPIO.port[port].pmr0c = 1 << portpin;
		    AVR32_GPIO.port[port].pmr1c = 1 << portpin;
		    AVR32_GPIO.port[port].pmr2s = 1 << portpin;
		    break;
		case 'f':	
		    AVR32_GPIO.port[port].pmr0s = 1 << portpin;
		    AVR32_GPIO.port[port].pmr1c = 1 << portpin;
		    AVR32_GPIO.port[port].pmr2s = 1 << portpin;
		    break;					
	}
		//weise Pin die ber den Parameter 'func'
		//bergebene Funktion zu
}

void gpio_enable_pullup(uint8_t port, uint8_t portpin)
{
	//Aktiviere Pull-Up Widerstand
	AVR32_GPIO.port[port].puers = 1 << portpin;
}

void gpio_enable_interrupt(uint8_t port, uint8_t portpin, 
char mode) 
{
	//Aktiviere Interrupt eines Pins im gewnschten Modus
	//c=pin change,r=rising edge,f=falling edge
	switch(mode)
	{
		case 'c':	
			AVR32_GPIO.port[port].imr0c = 1 <<portpin;
			AVR32_GPIO.port[port].imr1c = 1 <<portpin;
			break;
		case 'r':
			AVR32_GPIO.port[port].imr0s = 1 <<portpin;
			AVR32_GPIO.port[port].imr1c = 1 <<portpin;
			break;
		case 'f':
			AVR32_GPIO.port[port].imr0c = 1 <<portpin;
			AVR32_GPIO.port[port].imr1s = 1 <<portpin;
			break;
	}
	//aktiviere Interrupt
	AVR32_GPIO.port[port].iers = 1 << portpin;
}

void gpio_disable_interrupt(uint8_t port, uint8_t portpin)
{
	//Deaktiviere Interrupt eines Pins
	AVR32_GPIO.port[port].ierc = 1 << portpin;
}

void gpio_clear_interrupt_flag(uint8_t port, uint8_t portpin)
{
	//beendet die Interruptanfrage durch 
	//Lschen des entsprechenden Bits
	AVR32_GPIO.port[port].ifrc = 1 << portpin;
}

/*+++++Hilfsroutinen des AST+++++*/
void ast_interrupt_enable()
{
	AVR32_AST.IER.per0 = 1; 
		//aktiviere periodischen Interrupt Per0
}

void ast_interrupt_disable()
{
	AVR32_AST.IDR.per0 = 1; 
		//deaktiviere den periodischen Interrupt Per0
}

void ast_clear_interrupt_flag()
{
	//beendet die Interruptanfrage durch lschen 
	//des entsprechenden Bits
	while(AVR32_AST.SR.busy);
	AVR32_AST.SCR.per0=1;
}







//___________________________
/*+++++CAN-Controller+++++*/
//___________________________
//CAN Message Definition: Tx Message
void messageblock(uint32_t ident, uint32_t mask, U64 can_data)
{
	/*can_msg_t tx_msg = {
		{
			{
				.id = ident,        // Identifier
				.id_mask  = mask,   // Mask
			},
		},
		.data.u64 = can_data,		// Data Response
	};*/

	
	
}

// CAN Message Definition: Rx Message
can_msg_t rx_msg = {
	{
		{
			.id = 0,                // Identifier
			.id_mask  = 0,          // Mask
		},
	},
	.data.u64 = 0x0LL,              // Data
};

can_msg_t tx_msg = {
	{
		{
			.id = 4,        // Identifier
			.id_mask  = 112,   // Mask
		},
	},
	.data.u64 = 0x000002,		// Data Response
};



can_mob_t rx_mob_msg = {
	CAN_MOB_NOT_ALLOCATED,          // Handle: by default CAN_MOB_NOT_ALLOCATED
	&rx_msg,						// Pointer on CAN Message
	8,                              // Data length DLC
	CAN_DATA_FRAME,                 // Request type : CAN_DATA_FRAME or CAN_REMOTE_FRAME
	CAN_STATUS_NOT_COMPLETED        // Status: by default CAN_STATUS_NOT_COMPLETED
};

can_mob_t tx_mob_msg = {
	CAN_MOB_NOT_ALLOCATED,      // Handle: by default CAN_MOB_NOT_ALLOCATED
	&tx_msg,					// Pointer on CAN Message
	8,                          // Data length DLC
	CAN_DATA_FRAME,             // Request type : CAN_DATA_FRAME or CAN_REMOTE_FRAME
	CAN_STATUS_NOT_COMPLETED    // Status: by default CAN_STATUS_NOT_COMPLETED
};




//Call Back called channel0 TX
void can_out_callback_channel0_tx(uint8_t handle, uint8_t event)
{
	//Transmission
	can_mob_free(1,handle);
	tx_mob_msg.status = event;
}

//Call Back called channel0 RX
void can_out_callback_channel0_rx(uint8_t handle, uint8_t event)
{
	//Receive
	rx_mob_msg.can_msg->data.u64 = can_get_mob_data(1,handle).u64;
	rx_mob_msg.can_msg->id = can_get_mob_id(1,handle);
	rx_mob_msg.dlc = can_get_mob_dlc(1,handle);
	rx_mob_msg.status = event;
	can_mob_free(1,handle);
}

//Prepare Data on CAN Bus
void can_example_prepare_data(uint8_t mode)
{
	if (mode==0)
	{
		usart_write_char(&AVR32_USART2, 5);
		//Prepare Transmission Mode
		//Initialize channel 1
		can_init(1, ((uint32_t)&mob_ram_ch0[0]), CANIF_CHANNEL_MODE_NORMAL,
		can_out_callback_channel0_tx);
		usart_write_char(&AVR32_USART2, 6);
		//Allocate one mob for TX
		tx_mob_msg.handle = can_mob_alloc(1);
		delay_s(100);
		usart_write_char(&AVR32_USART2, 7);
		//Check return if no mob are available
		if (tx_mob_msg.handle==CAN_CMD_REFUSED)
		{
			gpio_set(portc, LED_Y1);
			while(1);
		}
		delay_s(100);
		usart_write_char(&AVR32_USART2, 8);
		CANIF_set_data(1,0,0x1020);
		can_tx(1, tx_mob_msg.handle, tx_mob_msg.dlc, tx_mob_msg.req_type,
		tx_mob_msg.can_msg);
		
		//usart_write_char(&AVR32_USART2, (int) tx_msg.data);
		CANIF_disable(1);
		
		//tx_msg.id = tx_msg.id++;
	}
	else if (mode==1)
	{
		//Prepare Receive Mode 
		//Initialize channel 1 
		can_init(1, ((uint32_t)&mob_ram_ch0[0]), CANIF_CHANNEL_MODE_NORMAL,
		can_out_callback_channel0_rx);
		//Allocate one mob for RX 
		rx_mob_msg.handle = can_mob_alloc(1);
		can_rx(1, rx_mob_msg.handle, rx_mob_msg.req_type,
		rx_mob_msg.can_msg);
	}
	
}


//_______________________________________________________________
// Edit: 2015_01_28 -	Vereinfachung der Interruptroutine 
//						und SOC,SOH,SOF Berechnung

//+++Funktionsdeklaration+++

void data_to_CAN(uint8_t mode_can);









void data_to_CAN(uint8_t mode_can)
{
	if (mode_can==0)
	{
		
		//CAN-Nachricht senden
		//messageblock(0x123,0x003,0x134679AB);
		can_example_prepare_data(0);
			
		if (tx_mob_msg.status == CAN_STATUS_COMPLETED)
		{
			usart_write_line(&AVR32_USART2,"OK");
		}
		else if (tx_mob_msg.status == CAN_STATUS_ERROR)
		{
			usart_write_line(&AVR32_USART2,"ERROR");
			}
		else if(tx_mob_msg.status == CAN_STATUS_BUSOFF)
		{
			usart_write_line(&AVR32_USART2,"BUSOFF");
		}
		else if(tx_mob_msg.status == CAN_STATUS_NOT_COMPLETED)
		{
			usart_write_line(&AVR32_USART2,"NOT OK");
		}
		can_out_callback_channel0_tx(0,0);
	}
	else if (mode_can==1)		
	{
		//CAN-Nachricht empfangen
		can_example_prepare_data(1);
			
		if (rx_mob_msg.status == CAN_STATUS_COMPLETED)
		{
			usart_write_line(&AVR32_USART2,"EOK");
		}
		else if (rx_mob_msg.status == CAN_STATUS_ERROR)
		{
			usart_write_line(&AVR32_USART2,"EERROR");
		}
		else if(rx_mob_msg.status == CAN_STATUS_BUSOFF)
		{
			usart_write_line(&AVR32_USART2,"EBUSOFF");
		}
		else if(rx_mob_msg.status == CAN_STATUS_NOT_COMPLETED)
		{
		usart_write_line(&AVR32_USART2,"ENOT OK");
		}
		can_out_callback_channel0_rx(0,0);
	}
}



//______________________________________________________
/*+++++Hauptprogramm+++++*/
//______________________________________________________
int main(void)
{
	global_interrupt_disable();
	//Initialisierung des Boards
    board_init();
	//Initialisierung des Microcontrollers 
	mcu_init();											
 	//Warte auf die Initialisierung des Messchips
	wait_cycles(160000);
	gpio_set(portc, LED_G1);
					
	usart_write_char(&AVR32_USART2, 1);
	delay_s(10);
	usart_write_char(&AVR32_USART2, 2);
	//Konfigurierung des Interruptcontrollers 
	//mithilfe von ASF-Routinen
	INTC_init_interrupts();
	
	usart_write_char(&AVR32_USART2, 3);
	
	//INTC_register_interrupt(&ast_int_handler,
	//AVR32_AST_ALARM_IRQ+3,AVR32_INTC_INTLEVEL_INT0);
	
	//Registrierung der Interrupts
	//global_interrupt_enable();
	
	
	
	while(1) 
    {
		
		usart_write_char(&AVR32_USART2, 99);
		delay_s(300);
		data_to_CAN(0);
		delay_s(100);
		
		
	//Endlosschleife
    }	
}